import { from } from 'rxjs';
import { assertId, assertName, isArray, microtask } from './util';
export class JsonRxServer {
    constructor({ send, call, notify, maxActiveSubscriptions = 30 }) {
        this.active = new Map();
        this.send = send;
        this.call = call;
        this.notify = notify;
        this.maxActiveSubscriptions = maxActiveSubscriptions;
    }
    sendError(id, error) {
        const message = [-1, id, error];
        this.send(message);
    }
    onSubscribe(message) {
        const [id, name, payload] = message;
        assertId(id);
        assertName(name);
        try {
            if (this.active.size >= this.maxActiveSubscriptions)
                return this.sendError(id, { message: 'Too many subscriptions.' });
            const observable = from(this.call(name, payload));
            const ref = { buffer: [] };
            const subscription = observable.subscribe((data) => {
                if (data === undefined)
                    return;
                ref.buffer.push(data);
                microtask(() => {
                    if (!ref.buffer.length)
                        return;
                    try {
                        for (const payload of ref.buffer)
                            this.send([-2, id, payload]);
                    }
                    finally {
                        ref.buffer = [];
                    }
                });
            }, (error) => {
                const data = error instanceof Error ? { message: error.message } : error;
                const message = [-1, id, data];
                this.send(message);
            }, () => {
                try {
                    if (!ref.buffer.length)
                        return this.send([0, id]);
                    const last = ref.buffer.length - 1;
                    for (let i = 0; i <= last; i++) {
                        const isLast = i === last;
                        const message = isLast ? [0, id, ref.buffer[i]] : [-2, id, ref.buffer[i]];
                        this.send(message);
                    }
                }
                finally {
                    ref.buffer = [];
                }
            });
            this.active.set(id, subscription);
        }
        catch (error) {
            this.sendError(id, error instanceof Error ? { message: error.message } : error);
        }
    }
    onUnsubscribe(message) {
        const [, id] = message;
        assertId(id);
        const subscription = this.active.get(id);
        if (!subscription)
            return;
        this.active.delete(id);
        subscription.unsubscribe();
    }
    onNotification(message) {
        const [name, payload] = message;
        assertName(name);
        this.notify(name, payload);
    }
    onMessage(message) {
        if (!isArray(message))
            throw new Error('Invalid message');
        const [one] = message;
        if (typeof one === 'string')
            return this.onNotification(message);
        if (one > 0)
            return this.onSubscribe(message);
        if (one === -3)
            return this.onUnsubscribe(message);
        throw new Error('Invalid message');
    }
}
